
setwd("C:/Users/Basal/Documents/matrix_work")


error_matrix=function(file_name,output_name){
  
  
  ####Create a function calculate_OA to calculate overall accuracy when the confusion matrix is entered
  calc_OA = function(Conf_MAT){
    diag_elements  =diag(Conf_MAT)
    sum_diag_elements = sum(diag_elements)
    sum_CF = sum(Conf_MAT)
    OA  = (sum_diag_elements/sum_CF )*100 ##Calculate the overall accuracy
    OA = round(OA,2)
    return(OA)
  }
  ####Create a function calculate_UA to calculate user's accuracy accuracy when the confusion matrix is entered
  calc_UA = function(Conf_MAT){
    output = (diag(Conf_MAT)/rowSums(Conf_MAT))*100 ###Calculate User Accuracy
    output = as.numeric(output) ### Convert the accuracies to Vector
    output = round(output,2)
    return(output)
  }
  ####Create a function calculate_PA to calculate producer's accuracy when the confusion matrix is entered
  calc_PA = function(Conf_MAT){
    output = (diag(Conf_MAT)/colSums(Conf_MAT))*100 ###Calculate User Accuracy
    output = as.numeric(output) ### Convert the accuracies to Vector
    output = round(output,2)
    return(output)
  }
  ####Create a function calculate_kappa to calculate kappa when the confusion matrix is entered
  calc_kappa=function(Conf_MAT){
    num = (sum(Conf_MAT)*sum(diag(Conf_MAT))) - sum((colSums(Conf_MAT)*rowSums(Conf_MAT))) ##Calculate numerator
    den = (sum(Conf_MAT)^2) - sum((colSums(Conf_MAT)*rowSums(Conf_MAT))) ##Calculate denominator
    output = num/den
    output = round(output,4)
    return(output)
  }
  
  
  #########Create error matrix with user and producer accuracies
  
  class_num=(as.numeric((read.csv(file_name,header=FALSE, nrows=1,))[,1])) ###reads how many classes there are
  classes=read.csv(file_name,header=FALSE,skip=(1),nrows=class_num)        ###creates a table of class names and associated values
  classes=classes[order(classes$V1),]
  ref_table=read.csv(file_name,skip=(class_num+1), na.strings=c("","NA"))  ###pulls out the map and reference data for the matrix
  
  tab=table(ref_table$Ref,ref_table$Map, dnn=c("Map","Reference"))         ###generate the inital error matrix
  colnames(tab) = classes$V2[which(classes$V1 %in% colnames(tab))]
  rownames(tab) = classes$V2[which(classes$V1 %in% rownames(tab))]
  
  accuracy.tab = cbind(tab,rowSums(tab))                                   ###append row totals to matrix
  accuracy.tab = as.table(accuracy.tab)
  ncol = length(accuracy.tab[1,])
  colnames(accuracy.tab)[ncol]="Row Totals"
  
  accuracy.tab = rbind(accuracy.tab,c(colSums(tab)," "))                   ###append column totals to matrix
  accuracy.tab = as.table(accuracy.tab)
  nrow = length(accuracy.tab[,1])
  rownames(accuracy.tab)[nrow]="Column Totals"
  
  accuracy.tab = cbind(accuracy.tab,append(calc_UA(tab)," "))              ###append user's accuracies to matrix
  accuracy.tab = as.table(accuracy.tab)
  ncol = length(accuracy.tab[1,])
  colnames(accuracy.tab)[ncol]="Users.Accuracy(%)"
  
  accuracy.tab = rbind(accuracy.tab,append(calc_PA(tab),c(" "," ")))       ###append producer's accuracies to matrix
  accuracy.tab = as.table(accuracy.tab)
  nrow = length(accuracy.tab[,1])
  rownames(accuracy.tab)[nrow]="Producers.Accuracy(%)"
  
  names(attributes(accuracy.tab)$dimnames) <- c("Map","Reference")         
  
  oal = calc_OA(tab)                                                       ###calculate overall accuracy
  oal=as.table(oal)
  rownames(oal) = "Overall.Accuracy(%)"
  
  kl = calc_kappa(tab)                                                     ###calculate kappa
  kl=as.table(kl)
  rownames(kl) = "Overall.Kappa"
  
  results=list(accuracy.tab,oal,kl)                                        ###generate a list with final results
  
  
  #######Output a text file into the working directory
 
  sink(output_name)
  lapply(results,print)
  sink() 
 
  
  return(results)      ###the function returns a list with the error matrix, overall accuracy, and kappa as seperate elements
  
}

error_matrix("Ref_data.csv","Error_matrix.txt")